/* 
* Hardware.h
*
* Created: 11.06.2018 11:00:21
* Author: alex
*/


#ifndef __HARDWARE_H__
#define __HARDWARE_H__

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdint.h>
#include "Servo.h"
#include "Time.h"
#include "Uart.h"

#define PHOTO PD2
#define HALL PD3
#define TRIGGER PD4
#define SWITCH PD5
#define BB_LED PD7
#define SERVO PB1
#define BUTTON1 PB2
#define BUTTON2 PB3
#define LED1 PB4
#define LED2 PB5

class Hardware
{
//variables
public:
	static void (*photosensorCallback)(bool);
	static void (*hallsensorCallback)(bool);
	static void (*button1Callback)(bool);
	static void (*button2Callback)(bool);
	static void (*triggerCallback)(bool);
	static void (*switchCallback)(bool);

	static bool lastButton1State;
	static bool lastButton2State;
	static bool lastTriggerState;
	static bool lastSwitchState;
	
	static uint32_t lastButtonInterrupt;
	static uint32_t lastTriggerInterrupt;
protected:
private:


//functions
public:
	Hardware();
	~Hardware();
	
	static void init();
	static void attachPhotosensor(void (*f)(bool));
	static void attachHallsensor(void (*f)(bool));
	static void attachButton1(void (*f)(bool));
	static void attachButton2(void (*f)(bool));
	static void attachTrigger(void (*f)(bool));
	static void attachSwitch(void (*f)(bool));
	static void switchLED1(bool state);
	static void switchLED2(bool state);
	static uint32_t shot();

	static void emptyCallback(bool);
protected:
private:
	Hardware( const Hardware &c );
	Hardware& operator=( const Hardware &c );

}; //Hardware

#endif //__HARDWARE_H__
