/* 
* Servo.cpp
*
* Created: 11.06.2018 14:49:11
* Author: alex
*/


#include "Servo.h"

uint16_t Servo::interruptRounds = 0;

// default constructor
Servo::Servo()
{
} //Servo

void Servo::initServoTimer() {
	cli();
		
	TCNT2 = 0; // reset current timer-counter to 0
	TIMSK2 |= (1 << OCIE2A); // allow Compare Interrupt
	TCCR2A |= (1 << WGM21); // CTC Modus
	
	OCR2A = 249; // interrupt occurs every 16 ms
	
	sei();
}

uint32_t Servo::shot(uint32_t time) {
	TCNT1 = 0; // reset the counter so the servo adapts with constant time
	uint32_t timeToRelease = setAngle(SERVO_OPEN);	
	
	// start timer for closing servo
	TCNT2 = 0; // reset current timer-counter to 0
	TCCR2B |= (1 << CS22) | (1 << CS21) | (1 << CS20); // prescaler 1024
	interruptRounds = time/16; // interrupt occurs every 16 ms
	
	return timeToRelease;
}

uint32_t Servo::predictedTimeToRelease() {
	return (ICR1-TCNT1)/2; // calculate time to adaption of the servo
}

void Servo::servoTimerInterrupt() {
	if (!interruptRounds--) { // only when time is elapsed
		setAngle(SERVO_CLOSED);
		TCNT2 = 0; // reset current timer-counter to 0
		TCCR2B &= ~(1 << CS22) & ~(1 << CS21) & ~(1 << CS20); // timer over and out
	}
}

// default destructor
Servo::~Servo()
{
} //~Servo

void Servo::init(uint8_t angle) {
	// set Servo-Pin as output
	DDRB |= (1 << PB1);
	
	// set 20ms duty cycle
	ICR1 = 39999;
	
	// set none-inverting mode
	TCCR1A |= (1 << COM1A1)|(1 << COM1B1);
	
	// set Fast PWM mode using ICR1 as TOP
	TCCR1A |= (1 << WGM11);
	TCCR1B |= (1 << WGM12)|(1 << WGM13);
	
	setAngle(angle);
	
	// START the timer with /8 prescaler
	TCCR1B |= (1 << CS11);
	
	initServoTimer();
}

uint32_t Servo::setAngle(uint8_t angle) {
	OCR1A = calcDutyCycle(angle);
	return (ICR1-TCNT1)/2; // calculate time to adaption of the servo
}

uint16_t Servo::calcDutyCycle(uint8_t angle) {
	return (uint16_t)(3000.0/180.0 * (angle%180)) + 800; //1499;  // generate duty cycle between 0.75 (1499) and 2.25 ms (4499)
}

ISR(TIMER2_COMPA_vect) {
	Servo::servoTimerInterrupt();
}
