/** \file Time.cpp
 * Provides a millisecond accurately absolute system-time 
 * Uses the 8-bit Timer0 for counting
 *
 * \author Alexander Detsch
 * \date 03.06.2016 10:27:16
 * \version 1.0
 */

#include "Time.h"

volatile uint32_t Time::absMicros = 0;
void (*Time::taskCallback)();
uint32_t Time::taskTime = 0;
bool Time::taskFinished = true;

// default constructor
Time::Time() {	
	start(); // Start timer
} //Time

// default destructor
Time::~Time() {
	cli();
	stop(); // stop the timer
	absMicros = 0; // reset the time
	sei();
} //~Time

/**
@brief Initialization of Time

Initialize the timer and variables
*/
void Time::start() {
	cli();
	absMicros = 0; // Set the time to a defined 0
	
	TCNT0 = 0; // aktuellen Zhlerstand auf 0 setzen
	TIMSK0 |= (1<<OCIE0A); // Compare Interrupt erlauben
	TCCR0A |= (1<<WGM01); // CTC Modus
	//TCCR0B |= (1<<CS01); // Vorteiler 8
	TCCR0B |= (1<<CS01) | (1 << CS00); // Vorteiler 64
	//OCR0A = 199-1; // Nach 159 Schritten berlaufen (Index 0) --> Genau alle 100s
	OCR0A = 49-1; // nach 49 Schritten --> alle 200s
	sei();
}

/**
@brief Stops the Time

Stops the timer e.g. to reduce system load
*/
void Time::stop() {
	cli();
	TCCR0B &= ~(1<<CS00) | ~(1<<CS01) | ~(1<<CS02); // Timer deaktivieren
	sei();
}

/**
@brief Retrieve the current absolute system time.

@return the current system time in milliseconds
*/
uint32_t Time::millis() {
	return micros()/1000;
}

/**
@brief Retrieve the current absolute system time.

@return the current system time in microseconds
*/
uint32_t Time::micros() {
	unsigned long timeTemp = 0;
	
	//cli(); // to prevent an interrupt while reading the time
	timeTemp = absMicros;
	//sei();
	
	return timeTemp;
}

void Time::attachTask(void (*f)(), uint32_t timestamp) {
	taskCallback = f;
	taskTime = timestamp;
	taskFinished = false;
}

/**
@brief Increments the microsecond counter

static function for the timer to increment the microseconds
*/
void Time::timeInc() {
	absMicros+=200;
	
	if (!taskFinished && absMicros >= taskTime) {
		taskFinished = true;
		(*Time::taskCallback)();		
	}
}

ISR(TIMER0_COMPA_vect) {
	Time::timeInc();
}