/** \file Time.h
 * Provides a microsecond accurately absolute system-time 
 * Uses the 8-bit Timer0 for counting
 *
 * \author Alexander Detsch
 * \date 03.06.2016 10:27:16
 * \version 1.0
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stddef.h>
#include <inttypes.h>

#ifndef __TIME_H__
#define __TIME_H__


class Time
{
//variables
public:
	static volatile uint32_t absMicros; // storage for the absolute system time
protected:
private:
	static void (*taskCallback)();
	static uint32_t taskTime;
	static bool taskFinished;
	

//functions
public:
	Time();
	~Time();
	static void start(); // Start the timer. You dont have to do this.
	static void stop(); // Stop the timer. For whatever reason you will need this.
	static uint32_t millis(); // get the current time
	static uint32_t micros(); // get the current accurate time
	static void attachTask(void (*f)(), uint32_t timestamp);
	
	static void timeInc(); // Only for the ISR. Not for you!
	
protected:
private:
	Time( const Time &c );
	Time& operator=( const Time &c );

}; //Time

#endif //__TIME_H__
