/*
 * KIS-Kugelfall.cpp
 *
 * Created: 11.06.2018 11:36:00
 * Author : alex
 */ 

#include <avr/io.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <stdint.h>
#include <avr/sleep.h>
#include "lib/Hardware.h"
#include "lib/Time.h"
#include "lib/Uart.h"
#include "lib/InputProcessing.h"
#include "lib/Trigger.h"

#define OUTPUTMODUS 0 // 0: normal output, 1: speed
#define GOTOSLEEP 0

uint8_t pos = 0;

uint32_t shotStarted = 0;
uint32_t lastTriggerPressed = 0;

void triggerCallback(bool state) {
	PORTB ^= ( 1 << LED1 );
	Hardware::switchLED2(true);
	if (Time::millis() - lastTriggerPressed < 500) {
		Trigger::triggerPress(4); // we already planned one shot. So do four more
	} else {
		Trigger::triggerPress(1);
	}
	lastTriggerPressed = Time::millis();
	Trigger::planShot();
}

void button1(bool state) {
	triggerCallback(state);
}

void button2(bool state) {
	Trigger::resetVariableOffset();
}

void toggleLed() {
	Time::attachTask(toggleLed, Time::micros()+1000000);
	PORTB ^= ( 1 << LED1 );
}


int main(void) {
	Time::start();
	Uart::begin(115200, SERIAL_8N2);
	Hardware::init();
	
	//Uart::println("Hallo! :)");
	
	// attach to interrupts
	Hardware::attachTrigger(triggerCallback);
	Hardware::attachButton1(button1);
	Hardware::attachButton2(button2);
	Hardware::attachPhotosensor(InputProcessing::photoInput);
	Hardware::attachHallsensor(InputProcessing::hallInput);
	
	wdt_enable(WDTO_250MS);
    
	uint32_t last = 0;
    while (1) {
		wdt_reset();
		if (!Trigger::getLeftShots()) Hardware::switchLED2(false);
		

		if (OUTPUTMODUS == 0 && Time::millis() - last > 100) {
			Uart::print("\033[0;0H\e[?25l");
			Uart::print("Systemzeit:       ");
			Uart::print(Time::millis());
			Uart::print(" ms   \r\nGeschwindigkeit:  ");
			//Uart::println(1000000.0/(t_curr - t_lastH));
			Uart::print(InputProcessing::getSpeed());
			Uart::print(" U/s         \r\nverbleibend:      ");
			Uart::print(Trigger::getLeftShots());
			Uart::print("  \r\nnaechster Schuss: ");
			Uart::print(Trigger::getNextShotTime()/1000);
			//Uart::print(" ms  \r\nin:               ");
			//Uart::print(Trigger::getNextShotTime()/1000 - Time::millis());
			Uart::println(" ms                 \r\n                                 ");
			last = Time::millis();
		}
		
		if (OUTPUTMODUS == 1 && Time::millis() - last > 100) {
			Uart::print(Time::millis());
			Uart::print(";");
			Uart::println(InputProcessing::getSpeed());
			last = Time::millis();
		}
		
		if (GOTOSLEEP) {
			set_sleep_mode(SLEEP_MODE_IDLE);
			sleep_mode();
		}
    }
}
